/*	a r r a y c o n v s
	apl array conversion routines.
*/
#include <apl.h>
#include <support.h>
#define	REG	register
#define	UNS	unsigned
#define	UCHAR	unsigned char

extern	ARRAY	*domain(), *nonce();
static	ARRAY	*dest;
static	UNS	els;

/*	m k _ p t
	make an ARRAY, setting up dest (ptr), els (bnd).
*/
void	*mk_pt(eltype, rank, shape)
UNS	eltype, rank, *shape;
{
	els = arraybound(dest = mkarray(eltype, rank, shape));
	return arraystart(dest);
}

/*	a g e t _ a
	make copy of array and return pointer to copy.
*/

ARRAY	*aget_a(src)
REG ARRAY	*src;
{
	REG		*from, *to;
	REG UNS		i;
	
	from = (int *)src;
	i = src->length;
	dest = (ARRAY *)malloc(i*sizeof(int));
	
	for(to = (int *)dest; i--; )
		*to++ = *from++;

	return(dest);
}

ARRAY	*sget_b(src) // get SINT array from BOOL array. 
ARRAY	*src;
{
	REG unsigned char *from;
	REG signed char	*to;
	REG UNS		bnd;
	REG UCHAR	boolmask;

	from = arraystart(src);
	to = mk_pt(SINT, src->rank, src->shape);
	bnd = els;

	for(--from, boolmask = 0; bnd--;)
		{
		if(! (boolmask >>= 1))
			{
			boolmask = 0x80;
			++from;
			}
		*to++ = 0 != (boolmask & *from);
		}
	return(dest);
}

ARRAY	*iget_b(src)	// get INTG array from BOOL array. 
ARRAY	*src;
{
	REG unsigned char	*from;
	REG short	*to;
	REG UNS		bnd;
	REG UCHAR	boolmask;

	from = arraystart(src);
	to = (short *)mk_pt(INTG, src->rank, src->shape);
	bnd = els;
	
	for(--from, boolmask = 0; bnd--;)
		{
		if(! (boolmask >>= 1))
			{
			boolmask = 0x80;
			++from;
			}
		*to++ = 0 != (boolmask & *from);
		}
	return(dest);
}

ARRAY	*lget_b(src) // get LONG array from BOOL array. 
ARRAY	*src;
{
	REG unsigned char *from;
	REG long	*to;
	REG UNS		bnd;
	REG UCHAR	boolmask;

	from = arraystart(src);
	to = (long *)mk_pt(LONG, src->rank, src->shape);
	bnd = els;
	
	for(--from, boolmask = 0; bnd--;)
		{
		if(! (boolmask >>= 1))
			{
			boolmask = 0x80;
			++from;
			}
		*to++ = 0 != (boolmask & *from);
		}
	return(dest);
}

ARRAY	*dget_b(src) // get DOUB array from BOOL array. 
ARRAY	*src;
{
	REG UCHAR	*from;
	REGDOUB		*to;
	REG UNS		bnd;
	REG UCHAR	boolmask;

	from = (UCHAR *)arraystart(src);
	to = (double *)mk_pt(DOUB, src->rank, src->shape);
	bnd = els;

	for(--from, boolmask = 0; bnd--;)
		{
		if(! (boolmask >>= 1))
			{
			boolmask = 0x80;
			++from;
			}
		*to++ = 0 != (boolmask & *from);
		}
	return(dest);
}

ARRAY	*iget_s(src) // get INTG array from SINT array. 
ARRAY	*src;
{
	REG short	*to;
	REG UNS		bnd;
	REG signed char	*from;

	from = arraystart(src);
	to = (short *)mk_pt(INTG, src->rank, src->shape);
	bnd = els;

	while(bnd--) *to++ = (short)*from++;

	return(dest);
}

ARRAY	*lget_s(src) // get LONG array from SINT array. 
ARRAY	*src;
{
	REG UNS		bnd;
	REG long	*to;
	REG signed char	*from;

	from = arraystart(src);
	to = (long *)mk_pt(LONG, src->rank, src->shape);
	bnd = els;

	while(bnd--) *to++ = (long)*from++;

	return(dest);
}

ARRAY	*dget_s(src) // get DOUB array from SINT array. 
ARRAY	*src;
{
	REG signed char	*from;
	REGDOUB		*to;
	REG UNS		bnd;

	from = arraystart(src);
	to = (double *)mk_pt(DOUB, src->rank, src->shape);
	bnd = els;

	while(bnd--)
		*to++ = (double)*from++;

	return(dest);
}

ARRAY	*lget_i(src) // get LONG array from INTG array. 
ARRAY	*src;
{
	REG short	*from;
	REG long	*to;
	REG UNS		bnd;

	from = (short *)arraystart(src);
	to = (long *)mk_pt(LONG, src->rank, src->shape);
	bnd = els;

	while(bnd--) *to++ = (long)*from++;

	return(dest);
}

ARRAY	*dget_i(src) // get DOUB array from INTG array. 
ARRAY	*src;
{
	REG short	*from;
	REGDOUB		*to;
	REG UNS		bnd;

	from = (short *)arraystart(src);
	to = (double *)mk_pt(DOUB, src->rank, src->shape);
	bnd = els;

	while(bnd--)
		*to++ = (double)*from++;

	return(dest);
}

ARRAY	*dget_l(src) // get DOUB array from LONG array. 
ARRAY	*src;
{
	REG long	*from;
	REGDOUB		*to;
	REG UNS		bnd;

	from = (long *)arraystart(src);
	to = (double *)mk_pt(DOUB, src->rank, src->shape);
	bnd = els;

	while(bnd--) 
		*to++ = (double)*from++;

	return(dest);
}

ARRAY	*(*convfns[7][7])() =
{
	{aget_a, domain, domain, domain, domain, domain, nonce  },
	{domain, aget_a, sget_b, iget_b, lget_b, dget_b, nonce  },
	{domain, nonce , aget_a, iget_s, lget_s, dget_s, nonce  },
	{domain, nonce , nonce , aget_a, lget_i, dget_i, nonce  },
	{domain, nonce , nonce , nonce , aget_a, dget_l, nonce  },
	{domain, nonce , nonce , nonce , nonce , aget_a, nonce  },
	{nonce , nonce , nonce , nonce , nonce , nonce , nonce  }
};
